data = ['Office Assistant II', 'Graphic Designer', 'Database Administrator III', 'Marketing Manager',
        'Computer Systems Analyst IV', 'Recruiter', 'Librarian', 'Food Chemist', 'VP Marketing',
        'Community Outreach Specialist', 'Sales Associate', 'Human Resources Assistant IV', 'Senior Sales Associate',
        'Help Desk Operator', 'Speech Pathologist', 'Occupational Therapist', 'Technical Writer', 'Analyst Programmer',
        'Research Assistant III', 'Quality Control Specialist', 'Junior Executive', 'Actuary', 'Health Coach III',
        'Cost Accountant', 'Internal Auditor', 'Dental Hygienist', 'Design Engineer', 'Accounting Assistant I',
        'Web Developer II', 'Editor', 'Web Designer I', 'Accounting Assistant I', 'Automation Specialist I',
        'Account Coordinator', 'Programmer Analyst I', 'Help Desk Operator', 'Social Worker', 'Senior Editor',
        'Registered Nurse', 'Safety Technician III', 'Internal Auditor', 'Nurse Practicioner',
        'Human Resources Manager', 'Assistant Manager', 'Research Nurse', 'VP Sales', 'Programmer Analyst I',
        'Accountant I', 'Clinical Specialist', 'Human Resources Manager', 'Editor', 'Data Coordiator',
        'Computer Systems Analyst II', 'Tax Accountant', 'Accountant II', 'Social Worker', 'Senior Sales Associate',
        'Accounting Assistant IV', 'Director of Sales', 'Senior Quality Engineer', 'Payment Adjustment Coordinator',
        'Paralegal', 'Actuary', 'Environmental Tech', 'Structural Engineer', 'Design Engineer',
        'Quality Control Specialist', 'Speech Pathologist', 'Quality Control Specialist', 'Help Desk Technician',
        'Software Engineer IV', 'VP Accounting', 'Account Executive', 'Desktop Support Technician', 'Junior Executive',
        'Financial Advisor', 'Database Administrator III', 'Developer I', 'Food Chemist', 'Information Systems Manager',
        'Executive Secretary', 'Cost Accountant', 'Director of Sales', 'Financial Analyst', 'Librarian',
        'Statistician III', 'Web Developer III', 'Structural Analysis Engineer', 'Desktop Support Technician',
        'Community Outreach Specialist', 'Research Assistant II', 'Assistant Professor', 'Safety Technician IV',
        'VP Product Management', 'Junior Executive', 'Accountant II', 'Design Engineer', 'Nuclear Power Engineer',
        'Operator']
